// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Ds9.h:	--- wrapper class around Ds9
//
//	DESCRIPTION:
//		A class `Ds9' is defined which implements the `ImageDisplayer'
//		interface using the image display program Ds9.
//
//	CHANGE HISTORY:
//		UL, 1999-05-06: 1.0.0: - first version
//		UL, 1999-12-15: 1.1.0: - added functionality to convert ds9 region
//								 file string into selectlib Region object
//		UL, 2002-09-04: 1.2.0: - added stopViewer argument to Ds9 constructor
//		UL, 2002-09-18: 1.3.0: - various additions (e.g. display region
//								 from GeoRegion object)
//		UL, 2002-11-13: 1.4.0: - added Ds9::deleteRegions() for global
//								 deletion of all markers in active ds9
//
//	AUTHOR:
//		U. Lammers (UL), Thu May  6 11:14:42 MET DST 1999
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/imgdisplay/src/Ds9.h,v 1.17 2013/04/02 18:18:05 aibarra Exp $
//

#ifndef DS9_H
#define DS9_H

#include <iostream>
#include "ImageDisplayer.h"
#include "Region.h"

class Ds9 : public virtual ImageDisplayer {
public:
	Ds9(bool stopViewer = false);
		//
		//	PURPOSE:
		//		Start up new ds9 process with title <name> if such a process
		//		does not exist already; stopViewer=true supresses the
		//		generation of a new process or brings down any existing
		//		process
		//


	bool isUp() const { return isRunning(); }

	void flipX() const;
	void flipY() const;

	void zoomIn(ZoomFactor zoomFactor = X1) const;
	void zoomOut(ZoomFactor zoomFactor = X1) const;

	void displayImage(const std::string &dataSetName) const;
	void displayRegion(const std::string &regionTable,
					   CoordType coordType = IMAGE) const;
	void displayRegion(const Region &region) const;

	std::vector<std::string> regionSelectionAsString(
											CoordType coordType = IMAGE) const;
	::GeoRegion regionSelection(CoordType coordType = IMAGE,
							const std::string &ignoreMarkerColor = "") const;

	static ::GeoRegion toRegion(const std::vector<std::string> &regionSpec,
								const std::string &ignoreMarkerColor = "", const bool wcsFlag = false);
		//
		//	PURPOSE:
		//		Convert a string vector with region specifications into
		//		a <GeoRegion> object
		//
		//	ARGUMENTS:
		//		regionSpec: vector with regions specification strings
		//					in Ds9 v3.0 format, e.g.
		//					"-circle(348.5,359,44) # text={yyy/zzz} color=red"
		//		ignoreMarkerColor:
		//					a string that must be a valid Ds9 region marker
		//					color or empty; in the former case all markers
		//					with this color will be ignored, i.e., not
		//					be used in the region construction; this is
		//					to allow for selective import of regions
		//
		//	RETURNS:
		//		GeoRegion object corresponding to given <regionSpec> string;
		//		all elements are packed into one Component; the shape objects
		//		of the components are dynamically created - ownership is
		//		transfered to the caller who is responsible for their
		//		deletion (use GeoRegion::free() for this purpose)
		//

	void deleteRegions() const;

	void send(const std::string &command) const;
	std::vector<std::string> receive(const std::string &command) const;

	void down() const;

	~Ds9();

private:
	static const std::string Xpaset;	//	send commands to Ds9
	static const std::string Xpaget;	//	read settings from Ds9
	static const std::string TITENV;	//	"Ds9-$LOGNAME"

	//	methods
	bool isRunning() const;
		//	Is Ds9 running?

	bool isThere() const;
		//	Is the executable there?

	void execBackground(const std::string &command) const;
		//	execute command in the background

	void setName();		//	figure out name from environment

	//	data members
	std::string	_title;	//	window title of Ds9
	std::string	_DS9;	//	name of the game

}; // Ds9;


std::ostream &operator<<(std::ostream &os,
						 const ImageDisplayer::Region &region);
	//
	//	PURPOSE:
	//		Write a region to an ostream in the Ds9 region file format
	//

std::istream &operator>>(std::istream &is, ImageDisplayer::Region &region);
	//
	//	PURPOSE:
	//		Read a region from an Ds9 region file istream
	//

#endif

