// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	imgdisplay.cc --- the imgdisplayer task
//
//	DESCRIPTION:
//		`imgdisplay' is used to display images in datasets using SAOtng.
//
//	CHANGE HISTORY:
//		UL, 1998-10-20: 0.1.0: - initial version
//		UL, 1999-02-11: 1.0.0: - added include statement enforced by
//								 taskmain >v1.17
//		UL, 1999-11-01: 1.1.0: - added "send"/"receive" command line arguments
//		UL, 2000-01-19: 1.2.0: - added support for two now parameters:
//								    withregiontable
//									regiontable
//
//	AUTHOR:
//		U. Lammers (UL), Tue Oct 20 15:42:15 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/imgdisplay/src/imgdisplay.cc,v 1.17 2009/08/31 15:15:17 aibarra Exp $
//
#include <iostream>
#include <string>

#include "param.h"
#include "errstr.h"
#include "SAOtng.h"
#include "Ds9.h"
#include "stringutil.h"
#include "SasImageDisplayer.h"

#include "imgdisplay.h"

void imgdisplay()
{
	if (booleanParameter("startviewer")) {
		ImageDisplayer *viewer;
		//
		//	figure which image viewer to use
		//
		std::string displ(stringParameter("viewer"));
		if (displ == "ENV") {
			//
			//	no viewer specified as task parameter - get value
			//	from environment
			//
			char *p = getenv(SAS_IMAGE_VIEWER_ENV_NAME);
			if (p)
				displ = p;
			else
				errstr<<"No image viewer specified via task parameter "
						"`viewer\' and environment variable `"
					  <<SAS_IMAGE_VIEWER_ENV_NAME
					  <<"\' not set either - cannot continue"
					  <<Sas::error("NoImageViewerSpecified");
		}
		//
		//	now instantiate the right viewer
		//
		std::string viewerstr(lowercase(displ));
		bool stopViewer = booleanParameter("stopviewer");
		if (viewerstr.find("saotng") != std::string::npos)
			viewer = new SAOtng(stopViewer);
		else
			viewer = new Ds9(stopViewer);

		if (viewer->isUp()) {
			if (booleanParameter("withimagefile"))
				viewer->displayImage(stringParameter("imagefile"));
			if (booleanParameter("flipx"))
				viewer->flipX();
			if (booleanParameter("flipy"))
				viewer->flipY();
			if (booleanParameter("withregion")) {
				setParameter("regions", viewer->regionSelectionAsString());
			}
			if (booleanParameter("withregiontable"))
				viewer->displayRegion(stringParameter("regiontable"));

			//	process additional commands
			if (booleanParameter("withcommands")) {
				std::vector<std::string>
									commands(stringListParameter("commands"));
				for (unsigned n=0; n<commands.size(); ++n)
					viewer->send(commands[n]);
			}
			if (booleanParameter("withreceive")) {
				std::vector<std::string>
									commands(stringListParameter("commands")),
								    reply;
				for (unsigned n=0; n<commands.size(); ++n) {
					std::vector<std::string> sv = viewer->receive(commands[n]);
					for (unsigned i=0; i<sv.size(); reply.push_back(sv[i++])) ;
				}
				setParameter("receive", reply);
			}
		}
	}
} // imgdisplay()
